#define PACKAGE_VERSION "2.5.0"
#define PACKAGE_STRING "libsrtp2 2.5.0"

#define GCM
#define OPENSSL
#define CPU_CISC 1
#define HAVE_INTTYPES_H 1
#define HAVE_STDINT_H 1
#define HAVE_STDLIB_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_UINT8_T 1
#define HAVE_UINT16_T 1
#define HAVE_UINT32_T 1
#define HAVE_UINT64_T 1
#define HAVE_INT32_T 1
